require 'spec_helper'

describe HonestPubsub::Server::ClientQueueListener do
  describe "#message_received" do

    subject { -> { queue_listener.message_received( {}, {routing_key: routing_key}, {context: context, payload: payload}) } }
    let(:queue_listener) { HonestPubsub::Server::ClientQueueListener.new(Klass, routing_key, 'queue') }
    let(:routing_key) { 'some.test.key' }
    let(:context) { {} }
    let(:payload) { {hello: 'moto'} }
    let(:context_before){}
    let(:validators){ [] }

    before do
      context_before
      Object.const_set :Klass, Class.new(HonestPubsub::Subscriber)
      validators.each do |validator|
        Klass.validates_payload_with validator
      end

      klass_instance

      allow(Klass).to receive(:new).and_return(klass_instance)
      subject.call unless respond_to? :no_call
    end

    after do
      Object.send :remove_const, :Klass
    end

    context "Normal execution" do
      let!(:klass_instance){
        instance = Klass.new({},{}, {})
        allow(instance).to receive(:perform)
        instance
      }

      context "test exception" do
        let(:no_call) { true }

        it { should_not raise_exception }
      end

      context "call" do
        it { expect(klass_instance).to have_received(:perform).with({}, payload) }
      end
    end

    context "Exception during execution" do
      let!(:klass_instance) {
        instance = Klass.new({},{}, {})
        allow(instance).to receive(:perform){
          raise StandardError.new("test")
        }
        instance
      }
      let(:no_call) { true }

      it { should_not raise_exception }
    end

    context "Payload error" do
      let!(:klass_instance) {
        instance = Klass.new({},{}, {})
        allow(instance).to receive(:perform)
        instance
      }

      let!(:validators) { [lambda{ |payload| false }]  }

      let(:context_before){ allow(Airbrake).to receive(:notify)}

      it {expect(Airbrake).to have_received(:notify)}
    end
  end
end