require 'spec_helper'

describe HonestPubsub::Logger do
  let(:routing_key) { "test/logger" }
  let(:subject) { HonestPubsub::Logger.new }
  let(:payload) { {"hit"=>"me"} }
  let(:message) { HonestPubsub::Message.new.serialize(context, routing_key, payload) }
  let(:config_buffer) { StringIO.new }
  let(:output_string) { config_buffer.string }
  let(:config_override) { {} }
  let(:context) { {unique_id: "1234", orig_ip_address: "127.0.0.1"}}

  before do
    HonestPubsub::Logging.instance_variable_set(:"@logger", nil)
    directory = File::dirname(__FILE__)
    full_name = File.join(directory, "config.yml")
    @config_data = HashWithIndifferentAccess.new( YAML.load_file(full_name)[ENV["RAILS_ENV"]] )
    @config_data.merge!(config_override)
    @config_data[:logger][:file] = config_buffer
    allow(HonestPubsub::Configuration).to receive(:configuration).at_least(1).and_return(@config_data)
  end

  describe "#log_publish" do
    let(:result) { subject.log_publish(routing_key, message)}


    it "should not fail" do
      expect{ result }.not_to raise_error()
    end

    it "should write a row to the log" do
      result
      expect(output_string.length).not_to eq(0)
    end

  end

  describe "#failed_publish" do
    let(:result) { subject.failed_publish(routing_key, {}, message)}

    context "warning log level" do

      it "should not fail" do
        expect{ result }.not_to raise_error()
      end

      it "should respect the log level of the file" do
        result
        expect(output_string.length).not_to eq(0)
      end
    end

    context "fatal log level" do
      let(:config_override) { { 'logger'=> { 'level'=>'fatal' } } }

      it "should not fail" do
        expect{ result }.not_to raise_error()
      end

      it "should respect the log level of the file" do
        result
        expect(output_string.length).to eq(0)
      end
    end
  end

  describe "#log_receive" do
    let(:result) { subject.log_receive(routing_key, message)}

    it "should not fail" do
      expect{ result }.not_to raise_error()
    end

    it "should respect the log level of the file" do
      result
      expect(output_string.length).not_to eq(0)
    end
  end

  describe "#log_service" do
    let(:service_name) { "test" }
    let(:result) { subject.log_service(service_name, log_level, message)}

    context "debugger log" do
      let(:log_level) { :debug }
      it "should not fail" do
        expect{ result }.not_to raise_error()
      end

      it "should respect the log level of the file" do
        result
        expect(output_string.length).to eq(0)
      end
    end

    context "warn log" do
      let(:log_level) { :warn }
      it "should not fail" do
        expect{ result }.not_to raise_error()
      end

      it "should respect the log level of the file" do
        result
        expect(output_string.length).not_to eq(0)
      end
    end

  end

end
