require 'spec_helper'

describe HonestPubsub::Message do
  let(:original) { {'data'=> 'found'} }
  let(:subject) { HonestPubsub::Message.new }
  let(:routing_key) { "test.message" }
  let(:context) { {unique_id: "1234", orig_ip_address: "127.0.0.1"}}

  describe "#serialize" do
    let(:result) { subject.serialize(context, routing_key, original)}

    it "should not fail" do
      expect{ result }.not_to raise_error()
    end

    it "should have all the header items" do
      expect(result[:ts]).not_to be_nil
      expect(result[:v]).not_to be_nil
      expect(result[:pub]).not_to be_nil
      expect(result[:payload]).not_to be_nil
    end

    it "should have the data in the payload" do
      expect(result[:payload][:data]).to eq('found')
    end

    it "should have the attributes readable" do
      result
      expect(subject.ts).not_to be_nil
      expect(subject.version).not_to be_nil
      expect(subject.pub).not_to be_nil
      expect(subject.payload).not_to be_nil
    end
  end

  describe "#parse" do
    let(:payload) { subject.serialize(context, routing_key, original).to_json }
    let(:result) { subject.parse(payload) }

    it "should not fail" do
      expect{ result }.not_to raise_error()
    end

    it "should have all the hash items" do
      expect(result[:ts]).not_to be_nil
      expect(result[:v]).not_to be_nil
      expect(result[:pub]).not_to be_nil
      expect(result[:payload]).not_to be_nil
    end

    it "should have the data in the payload" do
      expect(result[:payload][:data]).to eq('found')
    end

    it "should have the attributes readable" do
      result
      expect(subject.ts).not_to be_nil
      expect(subject.version).not_to be_nil
      expect(subject.pub).not_to be_nil
      expect(subject.payload).not_to be_nil
    end

  end

end
